"""
Module contains all schemas for settings
"""

from copy import deepcopy

from configs_core.schemas import CONFIGURATOR_USAGE as _CONFIGURATOR_USAGE, TYPE_BOOL, TYPE_INT, TYPE_SERVICE_ORIGIN

TYPE_NOT_NEGATIVE_INT = {"type": "integer", "minimum": 0}
TYPE_STR = {"type": "string"}
TYPE_FLOAT = {"type": "number"}
TYPE_POSITIVE_FLOAT = {"type": "number", "exclusiveMinimum": 0}
TYPE_PORT = {"type": "integer", "minimum": 1, "maximum": 65535}
TYPE_NATURAL = {"type": "integer", "minimum": 1}
UUID_PATTERN = "^[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}$"
TYPE_UUID = {"type": "string", "pattern": UUID_PATTERN}
DSN_PATTERN = (
    r"^(?:(?P<user>.*?)(?::(?P<passwd>.*))@)?(?P<net>[^\?\/]*)(?:\/(?P<dbname>[^\?]*)(?:\?(?P<params>[^\?]*))?)?$"
)
TYPE_DSN = {"type": "string", "pattern": DSN_PATTERN}

S3_SCHEMA = {
    "properties": {
        "host": TYPE_STR,
        "region": TYPE_STR,
        "aws_public_access_key": TYPE_STR,
        "aws_secret_access_key": TYPE_STR,
        "authorization_signature": {"type": "string", "enum": ["s3v2", "s3v4"]},
        "request_timeout": TYPE_INT,
        "connect_timeout": TYPE_INT,
        "verify_ssl": TYPE_BOOL,
    },
    "required": ["host", "aws_public_access_key", "aws_secret_access_key", "authorization_signature"],
    "type": "object",
    "additionalProperties": False,
}

LAMBDA_S3_SCHEMA = {
    "properties": {
        "host": TYPE_STR,
        "region": TYPE_STR,
        "aws_public_access_key": TYPE_STR,
        "aws_secret_access_key": TYPE_STR,
        "authorization_signature": {"type": "string", "enum": ["s3v2", "s3v4"]},
        "request_timeout": TYPE_INT,
        "connect_timeout": TYPE_INT,
        "bucket": TYPE_STR,
        "verify_ssl": TYPE_BOOL,
    },
    "required": ["host", "aws_public_access_key", "aws_secret_access_key", "authorization_signature", "bucket"],
    "type": "object",
    "additionalProperties": False,
}

DB_SETTINGS_SCHEMA = {
    "type": "object",
    "properties": {"connection_pool_size": TYPE_NATURAL},
    "required": ["connection_pool_size"],
    "additionalProperties": False,
}

DB_SCHEMA = {
    "properties": {
        "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
        "dsn": TYPE_DSN,
        "db_user": TYPE_STR,
        "db_password": TYPE_STR,
        "db_name": TYPE_STR,
        "db_host": TYPE_STR,
        "db_port": TYPE_PORT,
        "db_settings": DB_SETTINGS_SCHEMA,
    },
    "required": ["db_type", "db_settings"],
    "type": "object",
    "additionalProperties": False,
}

SERVICE_ADDRESS = {
    "type": "object",
    "properties": {"origin": TYPE_SERVICE_ORIGIN, "api_version": TYPE_INT},
    "required": ["origin", "api_version"],
    "additionalProperties": False,
}

LUNA_IMAGE_STORE_ADDRESS = {
    "type": "object",
    "properties": {**SERVICE_ADDRESS["properties"], "bucket": TYPE_STR},
    "required": [*SERVICE_ADDRESS["required"], "bucket"],
    "additionalProperties": False,
}

INFLUX_MONITORING_SCHEMA = {
    "properties": {
        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
        "use_ssl": {"type": "integer", "enum": [0, 1]},
        "flushing_period": {"type": "number"},
        "host": TYPE_STR,
        "port": TYPE_PORT,
        "organization": TYPE_STR,
        "token": TYPE_STR,
        "bucket": TYPE_STR,
    },
    "default": {
        "send_data_for_monitoring": 1,
        "use_ssl": 0,
        "flushing_period": 1,
        "host": "127.0.0.1",
        "port": 8086,
        "organization": "luna",
        "token": "secret-token",
        "bucket": "luna_monitoring",
    },
    "required": ["send_data_for_monitoring", "host", "port", "organization", "token", "bucket"],
    "type": "object",
    "additionalProperties": False,
    "description": "Monitoring settings.",
}

REDIS_SENTINEL = {
    "description": "Redis sentinel settings. Host+port",
    "type": "string",
    "pattern": r"[a-zA-Z0-9.]+:\d{1,5}",
    "example": "127.0.0.1:26379",
}

_REDIS_SENTINELS_SCHEMA = {
    "description": "Redis sentinels settings",
    "type": "object",
    "properties": {
        "master_name": TYPE_STR,
        "sentinels": {"type": "array", "items": REDIS_SENTINEL},
        "user": TYPE_STR,
        "password": TYPE_STR,
    },
    "additionalProperties": False,
    "required": ["master_name", "sentinels", "user", "password"],
}

_REDIS_DB_NUMBER = {"type": "integer", "enum": list(range(15))}
REDIS_DB_SCHEMA = {
    "type": "object",
    "properties": {
        "host": TYPE_STR,
        "port": TYPE_PORT,
        "user": TYPE_STR,
        "password": TYPE_STR,
        "channel": TYPE_STR,
        "sentinel": _REDIS_SENTINELS_SCHEMA,
    },
    "additionalProperties": False,
}

REDIS_PUBSUB_SCHEMA = {
    "type": "object",
    "properties": {"host": TYPE_STR, "port": TYPE_PORT, "password": TYPE_STR, "channel": TYPE_STR},
    "additionalProperties": False,
}

REDIS_STORAGE_SCHEMA = {
    "type": "object",
    "properties": {
        "host": TYPE_STR,
        "port": TYPE_PORT,
        "user": TYPE_STR,
        "password": TYPE_STR,
        "sentinel": _REDIS_SENTINELS_SCHEMA,
        "number": _REDIS_DB_NUMBER,
    },
    "additionalProperties": False,
}

LOGGER_SCHEMA = {
    "type": "object",
    "properties": {
        "log_level": {"type": "string", "enum": ["ERROR", "WARNING", "INFO", "DEBUG"]},
        "folder_with_logs": TYPE_STR,
        "max_log_file_size": TYPE_INT,
        "log_time": {"type": "string", "enum": ["LOCAL", "UTC"]},
        "log_to_stdout": TYPE_BOOL,
        "log_to_file": TYPE_BOOL,
        "multiline_stack_trace": TYPE_BOOL,
        "format": {"type": "string", "enum": ["default", "json", "ecs"]},
    },
    "additionalProperties": False,
}

AIOHTTP_TIMEOUT = {
    "type": "object",
    "properties": {
        "request": TYPE_POSITIVE_FLOAT,
        "connect": TYPE_POSITIVE_FLOAT,
        "sock_connect": TYPE_POSITIVE_FLOAT,
        "sock_read": TYPE_POSITIVE_FLOAT,
    },
    "description": "timeouts in seconds",
    "additionalProperties": False,
}

ADDITIONAL_SERVICES_USAGE = {
    "type": "object",
    "properties": {
        "luna_sender": TYPE_BOOL,
        "luna_events": TYPE_BOOL,
        "luna_tasks": TYPE_BOOL,
        "luna_matcher_proxy": TYPE_BOOL,
        "luna_image_store": TYPE_BOOL,
        "luna_handlers": TYPE_BOOL,
        "luna_lambda": TYPE_BOOL,
    },
    "required": [
        "luna_sender",
        "luna_events",
        "luna_tasks",
        "luna_matcher_proxy",
        "luna_image_store",
        "luna_handlers",
        "luna_lambda",
    ],
    "description": "whether to use nested optional services",
    "additionalProperties": False,
}

CONFIGURATOR_USAGE = deepcopy(_CONFIGURATOR_USAGE)

SDK_VIDEO_SETTINGS = {
    "type": "object",
    "properties": {
        "decoder_device_class": {"type": "string", "enum": ["cpu", "auto"]},
        "decoder_worker_count": TYPE_NATURAL,
        "storage": TYPE_STR,
        "max_size": TYPE_NATURAL,
    },
    "description": "sdk video processing settings",
    "additionalProperties": False,
}

RUNTIME_SETTINGS = {
    "type": "object",
    "properties": {
        "global_device_class": {"type": "string", "enum": ["cpu", "gpu"]},
        "num_threads": TYPE_NATURAL,
        "num_compute_streams": TYPE_NATURAL,
    },
    "description": "sdk runtime settings",
    "additionalProperties": False,
}

DETECTOR_TYPE = {"type": "string", "enum": ["FACE_DET_V3"]}

_FACE_DETECTOR_V3 = {
    "type": "object",
    "properties": {
        "min_face_size": TYPE_NATURAL,
        "redetect_tensor_size": TYPE_NATURAL,
        "redetect_face_target_size": TYPE_NATURAL,
        "redetect_score_threshold": TYPE_FLOAT,
        "score_threshold": TYPE_FLOAT,
    },
    "description": "settings for face detector version 3",
    "additionalProperties": False,
}

_BODY_DETECTOR = {
    "type": "object",
    "properties": {
        "image_size": TYPE_NATURAL,
        "redetect_score_threshold": TYPE_FLOAT,
        "score_threshold": TYPE_FLOAT,
        "landmarks17_threshold": TYPE_FLOAT,
    },
    "description": "settings for human body detector",
    "additionalProperties": False,
}

_FSDK_LIVENESS = {
    "type": "object",
    "properties": {
        "real_threshold": {"type": "number", "minimum": 0, "maximum": 1},
        "quality_threshold": {"type": "number", "minimum": 0, "maximum": 1},
    },
    "description": "FSDK one shot liveness settings",
    "additionalProperties": False,
}

_FSDK_DEEPFAKE = {
    "type": "object",
    "properties": {
        "real_threshold": {"type": "number", "minimum": 0, "maximum": 1},
        "estimation_mode": {"type": "integer", "enum": [1, 2]},
    },
    "description": "FSDK deepfake estimator settings",
    "additionalProperties": False,
}

_ESTIMATOR_RUNTIME_SETTINGS = {
    "type": "object",
    "properties": {
        "device_class": {"type": "string", "enum": ["global", "cpu", "gpu"]},
        "optimal_batch_size": TYPE_NATURAL,
        "worker_count": TYPE_NATURAL,
    },
    "additionalProperties": False,
}

SDK_ESTIMATOR_SETTINGS = {
    "type": "object",
    "properties": {"runtime_settings": _ESTIMATOR_RUNTIME_SETTINGS},
    "description": "settings for sdk estimator",
    "additionalProperties": False,
}

SDK_LIVENESS_ESTIMATOR_SETTINGS = {
    "type": "object",
    "properties": {
        "runtime_settings": _ESTIMATOR_RUNTIME_SETTINGS,
        "estimator_settings": _FSDK_LIVENESS,
    },
    "description": "settings for sdk liveness estimator",
    "additionalProperties": False,
}

SDK_DEEPFAKE_ESTIMATOR_SETTINGS = {
    "type": "object",
    "properties": {
        "runtime_settings": _ESTIMATOR_RUNTIME_SETTINGS,
        "estimator_settings": _FSDK_DEEPFAKE,
    },
    "description": "settings for sdk deepfake estimator",
    "additionalProperties": False,
}

SDK_FACE_DETECTOR_SETTINGS = {
    "type": "object",
    "properties": {
        "runtime_settings": _ESTIMATOR_RUNTIME_SETTINGS,
        "estimator_settings": _FACE_DETECTOR_V3,
    },
    "description": "settings for sdk face detector",
    "additionalProperties": False,
}

SDK_BODY_DETECTOR_SETTINGS = {
    "type": "object",
    "properties": {
        "runtime_settings": _ESTIMATOR_RUNTIME_SETTINGS,
        "estimator_settings": _BODY_DETECTOR,
    },
    "description": "settings for body detector",
    "additionalProperties": False,
}


ATTRIBUTES_STORAGE_POLICY = {
    "type": "object",
    "properties": {"default_ttl": TYPE_NOT_NEGATIVE_INT, "max_ttl": TYPE_NOT_NEGATIVE_INT},
    "description": "temporary attributes storage settings",
    "additionalProperties": False,
}

CACHE_MATCHING_SETTINGS = {
    "type": "object",
    "properties": {
        "thread_count": TYPE_NOT_NEGATIVE_INT,
        "tasks_count": TYPE_NOT_NEGATIVE_INT,
        "batch_size": TYPE_NOT_NEGATIVE_INT,
    },
    "description": "matcher lib settings",
    "additionalProperties": False,
}

RPC_SETTINGS = {
    "type": "object",
    "properties": {
        "timeouts": {
            "type": "object",
            "properties": {
                "connect_timeout": TYPE_NOT_NEGATIVE_INT,
                "request_timeout": TYPE_NOT_NEGATIVE_INT,
                "response_timeout": TYPE_NOT_NEGATIVE_INT,
            },
            "description": "matcher lib settings",
            "additionalProperties": False,
        },
        "pool_size": TYPE_NOT_NEGATIVE_INT,
    },
    "description": "matcher lib settings",
    "additionalProperties": False,
}

CACHED_LISTS_EXCLUDE = {
    "type": ["array", "null"],
    "items": TYPE_UUID,
    "maxItems": 1000,
    "description": "lists that should not be cached (set to null to disable)",
    "additionalProperties": False,
}

CACHED_LISTS_INCLUDE = {
    "type": ["array", "null"],
    "items": TYPE_UUID,
    "maxItems": 1000,
    "description": "lists that should be cached (set to null to disable)",
    "additionalProperties": False,
}

CACHED_DATA = {
    "type": "object",
    "properties": {
        "faces_lists": {
            "type": "object",
            "properties": {"exclude": CACHED_LISTS_EXCLUDE, "include": CACHED_LISTS_INCLUDE},
            "description": "faces lists cache rules",
            "additionalProperties": False,
        },
        "additionalProperties": False,
    },
    "description": "cached matching data",
    "additionalProperties": False,
}

DESCRIPTORS_CACHE = {
    "type": "object",
    "properties": {
        "cache_enabled": TYPE_BOOL,
        "updating_cache_interval": TYPE_POSITIVE_FLOAT,
        "matching_settings": CACHE_MATCHING_SETTINGS,
        "rpc_settings": RPC_SETTINGS,
        "cached_data": CACHED_DATA,
    },
    "description": "python matcher descriptor caching settings",
    "additionalProperties": False,
}

_MATCH_LIMITS = ["array_filter_limit", "reference_limit", "candidate_limit", "result_candidate_limit"]
_MATCH_LIMITS_SCHEMA = {
    "type": "object",
    "properties": dict(zip(_MATCH_LIMITS, [TYPE_NATURAL] * len(_MATCH_LIMITS))),
    "additionalProperties": False,
}

_CROSS_MATCH_LIMITS = ["short_array_filter_limit", "array_filter_limit", "result_candidate_limit", "general_limit"]
_CROSS_MATCH_LIMITS_SCHEMA = {
    "type": "object",
    "properties": dict(zip(_CROSS_MATCH_LIMITS, [TYPE_NATURAL] * len(_CROSS_MATCH_LIMITS))),
    "additionalProperties": False,
}

PLATFORM_LIMITS_SCHEMA = {
    "type": "object",
    "properties": {"match": _MATCH_LIMITS_SCHEMA, "cross_match": _CROSS_MATCH_LIMITS_SCHEMA},
    "additionalProperties": False,
}

_LIMITS_PROPS = [
    "received_images_limit",
    "raw_event_detections_limit",
    "raw_event_arrays_limit",
    "result_candidate_limit",
]
LIMITS_SCHEMA = {
    "type": "object",
    "properties": dict(zip(_LIMITS_PROPS, [TYPE_NATURAL] * len(_LIMITS_PROPS))),
    "additionalProperties": False,
}

_REMOTE_SDK_LIMITS = [
    "received_images_limit",
]

LUNA_REMOTE_SDK_LIMITS_SCHEMA = {
    "type": "object",
    "properties": dict(zip(_REMOTE_SDK_LIMITS, [TYPE_NATURAL] * len(_REMOTE_SDK_LIMITS))),
    "additionalProperties": False,
}

_HASP_CONFIG = {
    "type": "object",
    "properties": {"vendor": {"type": "string", "enum": ["hasp"]}, "server_address": {"type": "string"}},
    "additionalProperties": False,
    "default": {
        "vendor": "hasp",
        "server_address": "127.0.0.1",
    },
}
_GUARDANT_CONFIG = {
    "type": "object",
    "properties": {
        "vendor": {"type": "string", "enum": ["guardant"]},
        "server_address": {"type": "string"},
        "license_id": {"type": "string"},
    },
    "additionalProperties": False,
    "default": {
        "vendor": "hasp",
        "server_address": "127.0.0.1",
        "license_id": "1",
    },
}
LICENSE_VENDOR = {"oneOf": [_HASP_CONFIG, _GUARDANT_CONFIG]}

SERVICE_HTTP_SETTINGS = {
    "type": "object",
    "properties": {
        "request_timeout": TYPE_NATURAL,
        "response_timeout": TYPE_NATURAL,
        "request_max_size": TYPE_NATURAL,
        "keep_alive_timeout": TYPE_NATURAL,
    },
    "additionalProperties": False,
}

CLUSTER_CREDENTIALS = {
    "type": "object",
    "properties": {"host": TYPE_STR, "token": TYPE_STR, "certificate_path": TYPE_STR},
    "additionalProperties": False,
}

CLUSTER_LOCATION = {
    "type": "string",
    "enum": ["remote", "local", "internal"],
}


LUNA_API_PLUGINS_SETTINGS = {
    "type": "object",
    "properties": {},
    "additionalProperties": True,
}

METRICS = {
    "type": "object",
    "properties": {
        "enabled": TYPE_BOOL,
        "metrics_format": TYPE_STR,
        "extra_labels": {"type": "object", "properties": {}, "additionalProperties": True},
    },
    "additionalProperties": False,
}
